import pandas as pd
import numpy as np
from datetime import datetime, timedelta

from ovretl.tracking_utils.check_shipment_active import check_shipment_active


def test_check_shipment_active():
    quotation = pd.Series(data=["to_be_requoted", False], index=["shipment_status", "cancelled"])
    assert check_shipment_active(quotation, pd.DataFrame()) == False
    finished = pd.Series(data=["finished", False], index=["shipment_status", "cancelled"])
    assert check_shipment_active(finished, pd.DataFrame()) == False
    arrival_date_active = pd.Series(
        data=["in_progress", np.nan, np.nan, np.nan, datetime.now() + timedelta(days=5), np.nan, False],
        index=[
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert check_shipment_active(arrival_date_active, pd.DataFrame()) == True
    arrival_date_not_active = pd.Series(
        data=["booking_request", np.nan, np.nan, np.nan, datetime.now() + timedelta(days=11), np.nan, False],
        index=[
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert check_shipment_active(arrival_date_not_active, pd.DataFrame()) == False
    departure_date_active = pd.Series(
        data=["in_progress", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False],
        index=[
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert check_shipment_active(departure_date_active, pd.DataFrame())
    departure_date_not_active = pd.Series(
        data=["in_progress", np.nan, np.nan, datetime.now() - timedelta(days=5), np.nan, np.nan, False],
        index=[
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert not check_shipment_active(departure_date_not_active, pd.DataFrame())
    is_arrived = pd.Series(
        data=["in_progress", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False],
        index=[
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert not check_shipment_active(is_arrived, pd.DataFrame(data={"date_type": ["actual", "actual"]}))
    is_not_arrived_and_active = pd.Series(
        data=["in_progress", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False],
        index=[
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert check_shipment_active(is_not_arrived_and_active, pd.DataFrame(data={"date_type": ["actual", "estimated"]}))



import pandas as pd
import numpy as np
from pandas.util.testing import assert_frame_equal

from ovretl import add_shipowners_to_shipment


def test_add_shipowners_to_shipment():
    shipowners_asociations = pd.DataFrame(
        data={
            "shipment_id": [1, 1, 1, 1, 1, 3, 3],
            "role": [
                "departure_truck_freight",
                "departure_fees",
                "freight",
                "arrival_fees",
                "arrival_truck_freight",
                "freight",
                "departure_truck_freight",
            ],
            "name": ["MSC", "Morrisson", "MSC", "Fatton", "MSC", "Air France", "Everoad"],
        }
    )
    shipments_df = pd.DataFrame(data={"shipment_id": [1, 2],})
    result = add_shipowners_to_shipment(shipments_df, shipowners_asociations)
    result_should_be = pd.DataFrame(
        data={
            "shipment_id": [1, 2],
            "pickup_shipowner": ["MSC", np.nan],
            "departure_shipowner": ["Morrisson", np.nan],
            "freight_shipowner": ["MSC", np.nan],
            "arrival_shipowner": ["Fatton", np.nan],
            "delivery_shipowner": ["MSC", np.nan],
        }
    )
    assert_frame_equal(result, result_should_be)



import pandas as pd

from ovretl.prices_utils.sum_category_price import select_category_price


def calculate_shipment_prices(shipment: pd.Series, df_prices: pd.DataFrame) -> pd.Series:
    if df_prices is None or len(df_prices) == 0:
        return shipment
    df_prices = df_prices.groupby("category").sum()
    shipment["departure_truck_freight_price"] = select_category_price(
        df_prices, "departure_truck_freight", "price_in_eur"
    )
    shipment["departure_fees_price"] = select_category_price(df_prices, "departure_fees", "price_in_eur")
    shipment["freight_price"] = select_category_price(df_prices, "freight", "price_in_eur")
    shipment["arrival_fees_price"] = select_category_price(df_prices, "arrival_fees", "price_in_eur")
    shipment["arrival_truck_freight_price"] = select_category_price(df_prices, "arrival_truck_freight", "price_in_eur")
    shipment["insurance_price"] = select_category_price(df_prices, "insurance", "price_in_eur")
    shipment["turnover"] = (
        shipment["departure_truck_freight_price"]
        + shipment["departure_fees_price"]
        + shipment["freight_price"]
        + shipment["arrival_fees_price"]
        + shipment["arrival_truck_freight_price"]
        + shipment["insurance_price"]
    )
    shipment["customs_price"] = select_category_price(df_prices, "customs", "price_in_eur")
    shipment["other_price"] = select_category_price(df_prices, "other", "price_in_eur")
    shipment["vat_price"] = df_prices["vat_price_in_eur"].sum()
    return shipment


def calculate_margins(shipment: pd.Series, df_prices: pd.DataFrame, df_prices_initial: pd.DataFrame) -> pd.Series:
    if df_prices is None or len(df_prices) == 0 or df_prices_initial is None or len(df_prices_initial) == 0:
        return shipment
    df_prices = df_prices.groupby("category").sum()
    df_prices_initial = df_prices_initial.groupby("category").sum()

    shipment["initial_margin_without_insurance"] = (
        df_prices_initial["margin_price_in_eur"].sum()
        - select_category_price(df_prices_initial, key="margin_price_in_eur", category="insurance")
        - select_category_price(df_prices_initial, key="margin_price_in_eur", category="customs")
        - select_category_price(df_prices_initial, key="margin_price_in_eur", category="other")
    )
    shipment["initial_margin_insurance"] = select_category_price(
        df_prices_initial, key="margin_price_in_eur", category="insurance"
    )
    shipment["margin_without_insurance"] = (
        df_prices["margin_price_in_eur"].sum()
        - select_category_price(df_prices, key="margin_price_in_eur", category="insurance")
        - select_category_price(df_prices, key="margin_price_in_eur", category="customs")
        - select_category_price(df_prices, key="margin_price_in_eur", category="other")
    )
    shipment["margin_insurance"] = select_category_price(df_prices, key="margin_price_in_eur", category="insurance")
    return shipment


def calculate_shipment_purchase_prices(shipment: pd.Series, df_prices: pd.DataFrame) -> pd.Series:
    if df_prices is None or len(df_prices) == 0:
        return shipment
    df_prices = df_prices.groupby("category").sum()
    shipment["departure_truck_freight_purchase_price"] = select_category_price(
        df_prices, category="departure_truck_freight", key="purchase_price_in_eur"
    )
    shipment["departure_fees_purchase_price"] = select_category_price(
        df_prices, category="departure_fees", key="purchase_price_in_eur"
    )
    shipment["freight_purchase_price"] = select_category_price(
        df_prices, category="freight", key="purchase_price_in_eur"
    )
    shipment["arrival_fees_purchase_price"] = select_category_price(
        df_prices, category="arrival_fees", key="purchase_price_in_eur"
    )
    shipment["arrival_truck_freight_purchase_price"] = select_category_price(
        df_prices, category="arrival_truck_freight", key="purchase_price_in_eur"
    )
    shipment["insurance_purchase_price"] = select_category_price(
        df_prices, category="insurance", key="purchase_price_in_eur"
    )
    return shipment


def add_prices_to_shipments(shipment: pd.Series, find_most_relevant_prices) -> pd.Series:
    df_prices, prices_origin = find_most_relevant_prices(shipment["shipment_id"], shipment["proposition_id"])
    df_prices_initial, initial_prices_origin = find_most_relevant_prices(
        shipment["shipment_id"], shipment["proposition_id"], initial_proposition=True
    )
    shipment = calculate_shipment_prices(shipment=shipment, df_prices=df_prices)
    shipment = calculate_margins(shipment=shipment, df_prices=df_prices, df_prices_initial=df_prices_initial)
    shipment = calculate_shipment_purchase_prices(shipment=shipment, df_prices=df_prices)
    shipment["prices_origin"] = prices_origin
    return shipment



import pandas as pd

from ovretl.billings_utils.extract_billing_numbers import extract_billing_numbers
from ovretl.billings_utils.compute_billing_status import compute_billing_status


def add_billing_info(shipments_df: pd.DataFrame, billings_df: pd.DataFrame):
    shipments_df["billing_numbers"] = shipments_df.apply(
        lambda row: extract_billing_numbers(billings_df, row["shipment_id"]), axis=1
    )
    shipments_df["billing_status"] = shipments_df.apply(
        lambda row: compute_billing_status(billings_df, row["shipment_id"]), axis=1
    )
    return shipments_df
